package com.cms.spring.custom;
import java.io.IOException;  
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.core.io.Resource;
import org.springframework.web.servlet.handler.AbstractUrlHandlerMapping;

import com.cms.admin.controller.WelcomeController;

public class PropertyUrlHandlerMapping extends AbstractUrlHandlerMapping {
	public static Logger _log = Logger.getLogger(PropertyUrlHandlerMapping.class.getName()); 
	private Resource[] _arrFiles = null;
	private final Map<String, Object> _urlMap = new HashMap<String, Object>();

	public void setMappings(Resource[] mapFiles) {
		this._arrFiles = mapFiles;
	}

	public Map<String, Object> getUrlMap() {
		return this._urlMap;
	}

	public void initApplicationContext() throws BeansException {
		super.initApplicationContext();
		readProperties(this._arrFiles, this._urlMap);
		registerHandlers(this._urlMap);
	}

	protected final void readProperties(Resource[] arrFiles, Map<String, Object> urlMap) throws BeansException {
		String strUrl = null;
		Properties prop = null;
		Enumeration<?> enmKeys = null;
		if (arrFiles == null) {
			_log.info("Could not find the configuration files.");
			return;
		}
		try 
		{
			_log.info("Started reading the properties file");
			prop = new Properties();
			for (Resource res : arrFiles) 
			{
				prop.load(res.getInputStream());
				enmKeys = prop.propertyNames();
				while (enmKeys.hasMoreElements()) 
				{
					strUrl = (String) enmKeys.nextElement();
					urlMap.put(strUrl, prop.get(strUrl));
				}
				enmKeys = null;
				prop.clear();
			}
		} catch (IOException ex) {
			this._urlMap.clear();
			throw new BeanInitializationException("Could not load properties",ex);
		} 
		finally 
		{
			if (enmKeys != null) enmKeys = null;
			if (prop != null) 	prop = null;
			if (strUrl != null)	strUrl = null;
		}
	}

	protected final void registerHandlers(Map<String, Object> urlMap)throws BeansException {
		String url = null;
		Object handler = null;
		Iterator<String> it = null;
		label211:
		try {
			if (urlMap.isEmpty()) 
			{
				 break label211;
			}
			else 
			{
				it = urlMap.keySet().iterator();
				while (it.hasNext()) 
				{
					url = (String) it.next();
					handler = urlMap.get(url);
					if (!url.startsWith("/")) 
					{
						url = "/" + url;
					}
					if ((handler instanceof String)) {
						handler = ((String) handler).trim();
					}
					registerHandler(url, handler);
					handler = null;
					url = null;
				}
				it = null;
			}
		} 
		finally
		{
			if (it != null) it = null;
			if (handler != null)handler = null;
			if (url != null) url = null;
		}
	}
}
